<?php 

namespace Teknisa\DF\Modules\Entrada\Services;

/* Bibliotecas */
use Doctrine\ORM\EntityManager;
use Geral\Entities\Empresa;
use Geral\Entities\Inscrestad;
use NFePHP\Common\Certificate;
use NFePHP\Common\Configure\Configure;
use NFePHP\Common\Exception\RuntimeException;
use NFePHP\Common\Files\FilesFolders;
use NFePHP\NFe\Common\Config;
use NFePHP\NFe\Tools as NFe;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Model\Fiscal\Entities\Arqxmlnfe;
use Teknisa\Model\Fiscal\Entities\Notadestinada;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Fiscal\Entities\Lotenfe;
use Teknisa\Model\Geral\Entities\Paramempresa;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

/* Classes DF */
use Teknisa\DF\Modules\Entrada\Factories\ManifestacaoDestinatarioFactory;
use Teknisa\DF\Services\TransmissaoNfeService;
use Teknisa\DF\Services\XmlNFeService;
use Teknisa\DF\Services\NFe\ReturnNFe;

class ManifestacaoDestinatarioService {
    
    const CONTADOR_NOVOCODIGO_SINCRONIZACAO    = 'SINCRONIZACAONFE';
    const CONTADOR_NOVOCODIGO_NOTAMANIFESTACAO = 'NOTAMANIFESTACAO';
    
    protected $manifestacaoDestinatarioFactory;
    protected $novoCodigo;
    protected $entityManager;
    protected $environment;
    protected $transmissaoService;
    protected $quantidadeEventos;
    protected $quantidadeNotas;
    protected $importacaoPendente = array();
    protected $notasDownload;
    protected $sincronizadas;
    protected $notasEncontradas = array();
    protected $chaves = array();
    protected $notas = array();
    protected $notasInexistentes = array();
    protected $xmlsNota = array();
    protected $erros = [];
    
    public function __construct(ManifestacaoDestinatarioFactory $manifestacaoDestinatarioFactory, $novoCodigo, $entityManager, $environment, $fileServerLink) {
        $this->manifestacaoDestinatarioFactory = $manifestacaoDestinatarioFactory;
        $this->novoCodigo           = $novoCodigo;
        $this->entityManager        = $entityManager;
        $this->environment          = $environment;
        $this->masks = new Masks();
        $this->fileServerLink       = $fileServerLink;
        set_time_limit(0);
    }
    
    public function sincronizarNfeManifestacao($config, $params){
        $config = json_decode(json_encode($config));
        $object_config = $config;
        $cnpj  = $object_config->cnpj;
        $tpAmb = $object_config->tpAmb;
        $ie    = $object_config->ie;
        
        $check = $this->manifestacaoDestinatarioFactory->checkLastSinc(substr($cnpj, 0, 8), $tpAmb);
        if($check['BOCONSULTA']) throw new \Exception('LOCK_SINC_MANIFEST', $check['MINRESTANTES']);
        
        $connection = $this->entityManager->getConnection();
    	$tools = $this->getToolsNFe($ie, $cnpj, json_encode($config));
    	$this->nfeTools = $tools;
        
        if(empty($params['CDINSCESTA']) && !empty($ie)){
            $params['CDINSCESTA'] = $ie;
        }
        
        $ultNSU = 0;
        if($params["FORMAPESQUISA"] === "0") {
            $ultNSU = $this->manifestacaoDestinatarioFactory->maxNrNsu($cnpj, $tpAmb, 'T')["MAXNRNSU"];
        }
        $ultNSU = (!$ultNSU) ? 0 : $ultNSU;
        $ultNSU = str_pad($ultNSU, 15, "0", STR_PAD_LEFT);
        $maxNsuPConsulta = 0;

        $count  = 0; $this->quantidadeEventos = 0; $this->quantidadeNotas = 0; $this->notasDownload = 0; $this->sincronizadas = 0;
        $this->chaves = []; $this->notas = []; $this->notasEncontradas = []; $this->notasInexistentes = []; $this->importacaoPendente = [];
        $this->erros = [];
        $existeNotas = true;
        
        while($existeNotas === true) {
            try {
                $notas = [];
                $nrSincronizacaoNfe = $this->novoCodigo->getCodeByTableName(self::CONTADOR_NOVOCODIGO_SINCRONIZACAO.$cnpj, $this->environment->getNrOrgTrab());
                $this->manifestacaoDestinatarioFactory->insereHoraSincronizacao($cnpj, $nrSincronizacaoNfe, $this->environment->getCdOperador(), $ultNSU, $this->environment->getNrOrgTrab(), $tpAmb);
                
                $this->buscaNotasSefaz($ultNSU, 0, $cnpj);
                $notas = $this->aResposta;
                
                $connection->beginTransaction();
                $this->trataRetornoLoteSincronizacao($cnpj, $notas, $object_config, $params, '');
                $connection->commit();
            }
            catch(\Exception $e) {
                if($e->getMessage() == "Nenhum documento localizado.") {
                    if($count == 0) $connection->commit();
                    else            $connection->rollback();
                    
                    $existeNotas = false;
                } else {
                    if($connection->isTransactionActive()) $connection->rollback();
                    
                    if(strpos($e->getMessage(), "Certificate") !== false)
                        throw new \Exception("Certificado vencido. " . $e->getMessage());
                }
                $this->erros[] = $e->getMessage() . ' - '. $e->getCode();
            }

            if(isset($notas["ultNSU"]) && isset($notas["maxNSU"]) && isset($notas["cStat"])) {
               if($notas["cStat"] == 656) { // Rejeição 656: Consumo Indevido.
                  $existeNotas = false;
                  break;
               }
                else if($notas["ultNSU"] != $notas["maxNSU"]) {
                    $ultNSU = $notas["ultNSU"];
                    $count++;
                }
                else {
                    $existeNotas = false;
                    break;
                }
            }
            else {
                $existeNotas = false;
                break;
            }
        }
        
        if(isset($notas["cStat"]) && $notas["cStat"] != 656) { // Rejeição 656: Consumo Indevido.
            $this->atualizaNSUFaltante($cnpj, $tpAmb, $object_config, $params, '', '');
        }
        
        $this->selecaoNFManifestar($cnpj, $tpAmb, $object_config);
        $this->downloadAutomaticoXML($cnpj, $tpAmb);
        
        $chavesRetorno = array();
        foreach($this->chaves as $chave) {
            $chavesRetorno[] = $chave;
        }
        
        return array("CHAVES" => $chavesRetorno, "DADOS" => $this->calculaDadosSincronizacao($this->notas), "ERROS" => $this->erros);
    }
    
    public function montaConfigManifestacao($empresa, $organizacao, $inscricao, $cnpj) {
        $objEmpresa = $this->entityManager->getRepository('Geral\Entities\Empresa')->findOneBy(['cdempresa' => $empresa, 'nrorg' => $organizacao]);
       
        //$cnpj = $this->masks->removeMascaraInscricao($cnpj);
        
        $razaosocial = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($objEmpresa->getNmrazsocempr())));
        $nomefantasia = $razaosocial;
        if(!empty($inscricao)){
            if(strpos("ISENTO",strtoupper($inscricao)) || strpos("ISENTA", strtoupper($inscricao))){
                $ie = "ISENTO";
            }
            else{
                $ie = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($inscricao)));
            }
            $inscrestad = self::getInscricaoEstadual($empresa, $organizacao, $inscricao, $cnpj);
        }
        else{
            $inscrestad = $this->entityManager->getRepository('Geral\Entities\Inscrestad')->findBy(['cdempresa' => $empresa, 'nrinsjuriest' => $cnpj, 'nrorg' => $organizacao]);
            if (empty($inscrestad)) {
                throw new \Exception('Inscrição Estadual não Encontrada');
            }
            else{
                $ieComCertificados = array_filter($inscrestad, function($certRow){
                    return !empty($this->manifestacaoDestinatarioFactory->getCertificado($certRow->getCdinscesta(), $certRow->getNrinsjuriest()));
    		    });

    		    if(empty($ieComCertificados)){
    		        throw new \Exception('Certificado Digital não Encontrado');
    		    }
    		    else{
    		        $inscrestad = $ieComCertificados[0];
    		        $ie = $inscrestad->getCdinscesta();
    		    }
            }
        }
        
        /** @var Inscrestad $inscrestad */
        
        // 1-Produção 2-Homologação
        $tpAmb = (int) $inscrestad->getIdtpambientenfe();
        
        $siglaUF = $objEmpresa->getSgestado();
        
        $config = [
            //"atualizacao" => "2017-02-20 09:11:21",
            "tpAmb"     => $tpAmb,
            "razaosocial" => $nomefantasia,
            "siglaUF"   => $siglaUF,
            "cnpj"      => $cnpj = $this->masks->removeMascaraInscricao($cnpj),
            "schemes"   => "PL_009_V4",
            "versao"    => "4.00",
            "ie"        => $ie,//= $this->masks->removeMascaraInscricao($ie),
            "cdempresa" => $empresa,
            "nrorg"     => $this->environment->getNrOrgTrab()
            // "tokenIBPT" => "AAAAAAA",
            // "CSC" => "GPB0JBWLUR6HWFTVEAS6RJ69GPCROFPBBB8G",
            // "CSCid" => "000001",
            //"aProxyConf" => [
            //    "proxyIp" => "",
            //    "proxyPort" => "",
            //    "proxyUser" => "",
            //    "proxyPass" => ""
            //]
        ];
        $resp = Config::validate(json_encode($config));
        return $resp;
    }
    
    /**
     * Retorna inscricao estadual da empresa
     * @param $empresa
     * @param $cdinscestad
     * @param $organizacao
     * @return Inscrestad
     * @throws \Exception
     */
    private function getInscricaoEstadual($empresa, $organizacao, $inscricao, $cnpj){
        //$cnpj = $this->masks->removeMascaraInscricao($cnpj);
        
        $inscrestad = $this->entityManager->getRepository('Geral\Entities\Inscrestad')->findOneBy(
                        ['cdempresa' => $empresa, 'nrinsjuriest' => $cnpj, 'nrorg' => $organizacao, 'cdinscesta'=> $inscricao]);

        if (!($inscrestad instanceof Inscrestad)) throw new \Exception('Inscrição Estadual não Encontrada');

        return $inscrestad;
    }
    
    private function downloadAutomaticoXML($cnpj, $tpAmb) {
        $notasDownload = $this->manifestacaoDestinatarioFactory->getNotasDownloadAutomatico($cnpj, $tpAmb);
        
        if($notasDownload) {
    		try {
        		foreach($notasDownload as $nota) {
        		    $xml                    = new \DOMDocument();
                    $xml->preservWhiteSpace = false; //elimina espacos em branco
                    $xml->formatOutput      = false;
                    $xml->loadXML($nota['DSARQXMLNSUNFE']);
                    
                    $protNFe = $xml->getElementsByTagName('protNFe')->item(0);
        		    
        			if($protNFe) {
        			    $chave = $protNFe->getElementsByTagName('infProt')->item(0)->getElementsByTagName('chNFe')->item(0)->nodeValue;
        			    
        			    if($chave) {
    			            if($this->environment->getNrOrgTrab() != 1) {
    			                if(!isset($this->manifestacaoDestinatarioFactory->existeNotaRelac($chave)[0])) {
                    			    $this->manifestacaoDestinatarioFactory->zGravaFile($tpAmb, $chave.'.xml', $nota['DSARQXMLNSUNFE'], 'importar/', $chave);
                                    $this->manifestacaoDestinatarioFactory->atualizaNFeBaixado($nota['NRINSJURIEST'], $nota['IDAMBIENTENFE'], $nota['NRNSU'], $this->environment->getNrOrgTrab());
                			        $this->notasDownload++;
    			                }
                			}
                			else {
                			    $diretorioIntegracao = $this->manifestacaoDestinatarioFactory->getDiretorioIntegracao();
                			    
            			        if(isset($diretorioIntegracao[0])) {
            			            if($diretorioIntegracao[0]["DSDIRIMPARQXML"] != null) {
            			                if(!is_file($diretorioIntegracao[0]["DSDIRIMPARQXML"]."/$chave.xml")) {
                			                file_put_contents($diretorioIntegracao[0]["DSDIRIMPARQXML"]."/$chave.xml", $nota['DSARQXMLNSUNFE']);
                			                $this->notasDownload++;
            			                }
            			            }
            			        }
                			}
        			    }
        			}
    			}
    		} catch(\Exception $e) {
    	        $this->erros[] = $e->getMessage() . ' - '. $e->getCode();
    	    }
        }
    }
    
    private function selecaoNFManifestar($cnpj, $tpAmb, $config) {
        $manifestacoesRestantes = $this->manifestacaoDestinatarioFactory->selecaoNFManifestar($cnpj, $tpAmb);
        
        if(!empty($manifestacoesRestantes)){
            foreach($manifestacoesRestantes as $manifestacao){
                try{
                    $notaManif = array("NRACESSONFE" => $manifestacao['NRACESSONFE']);
    	            $this->manifestaNfe($notaManif, '210210', '', $config);
                }
                catch(\Exception $e) {
                    $this->erros[] = $e->getMessage() . ' - '. $e->getCode();
                }
            }
        }
    }

    public function preencheTabelasSincManifestacao($sincronizacao, $config){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($sincronizacao['aDoc'] as $doc) {
                
                $stInsJur = '';
                $NSU = $doc['NSU'];
                $ultNSU = $sincronizacao['ultNSU'];
                $xml                    = new \DOMDocument();
                $xml->preservWhiteSpace = false; //elimina espaÃ§os em branco
                $xml->formatOutput      = false;
                $xml->loadXML($doc['doc']);
                $notaComEvento = false;
                $chave = $xml->getElementsByTagName('chNFe')->item(0)->nodeValue;
                if(!empty($chave)){
                    $notadestinada = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notadestinada')->findOneBy(
                    array('nracessonfe' => $chave));
                    switch ($doc['schema']) {
                    	case 'resNFe_v1.01.xsd': 
                    	    if(!isset($this->notasEncontradas[$chave])) {
                    	        $this->notasEncontradas[$chave] = $chave;
                    	    }
                            $this->preencheNotadestinada($notadestinada, $config, $xml, $chave, $NSU, $ultNSU, false);
                            break;
                        case 'resNFe_v1.00.xsd':
                            if(!isset($this->notasEncontradas[$chave])) {
                    	        $this->notasEncontradas[$chave] = $chave;
                    	    }
                            $this->preencheNotadestinada($notadestinada, $config, $xml, $chave, $NSU, $ultNSU, false);
                            break;
                        case 'procEventoNFe_v1.00.xsd':
                        	$notaComEvento = true;
                        	$docEvento = $this->converteXMLParaArray($doc["doc"]);
                            $this->preencheEventoNotadestinada($notadestinada, $config, $xml, $chave, $docEvento, true);
                            break;
                        default:
                            break;
                    }
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function preencheNotadestinada($notadestinada, $config, $xml, $chave, $NSU, $ultNSU, $atualizaEvento){
        $nota = $this->converteXmlParaArray($xml->saveXML());

        $existeNotaDestinada = $this->manifestacaoDestinatarioFactory->existeNotaDestinada($nota["chNFe"]);
        $stInsJur = $xml->getElementsByTagName('CPF')->item(0) ? $xml->getElementsByTagName('CPF')->item(0)->nodeValue : $xml->getElementsByTagName('CNPJ')->item(0)->nodeValue;

        if(isset($existeNotaDestinada[0])){
            $this->atualizaNotaDestinada($chave, $nota);
        }
        else{
            $nrProxNotadestinada = $this->novoCodigo->getCodeByTableName(self::CONTADOR_NOVOCODIGO_NOTAMANIFESTACAO, $config->nrorg);
            $notadestinada = $this->incluirNotadestinada($nrProxNotadestinada, $chave, $nota, $NSU, $ultNSU, $stInsJur, $config);
        } 
        
        $this->manifestacaoDestinatarioFactory->deletaNotaDuplicada($chave, $config->cnpj);
    }

    private function incluirNotadestinada($nrProxNotadestinada, $chave, $nota, $NSU, $ultNSU, $stInsJur, $config) {
        $dtRecbto = date('d/m/Y H:i:s', strtotime(substr($nota["dhRecbto"], 0, 19)));
        $dtEmissao = date('d/m/Y', strtotime(substr($nota["dhEmi"], 0, 19)));
        
        $this->manifestacaoDestinatarioFactory->incluirNotadestinada($nrProxNotadestinada, $NSU, $chave, $stInsJur, $nota["xNome"],
                $nota["IE"], $dtEmissao, $nota["tpNF"], $nota["vNF"], $nota["digVal"], $dtRecbto, $nota["cSitNFe"],
                '0', '0', null, null, null, null, null, $config->tpAmb, null, 'AN', $config->ie, substr($chave, 25, 9), $config->cnpj, $ultNSU, $this->environment->getNrOrgTrab());
    }
    
    private function atualizaNotaDestinada($chave, $nota) {
        $dtRecbto = date('d/m/Y H:i:s', strtotime(substr($nota["dhRecbto"], 0, 19)));
        
        $this->manifestacaoDestinatarioFactory->atualizaNotaDestinada($nota["cSitNFe"], '', $dtRecbto, $chave);
    }

    private function preencheEventoNotadestinada($notadestinada, $config, $xml, $chave, $docEvento, $existeTpEvento){
        $nota = $this->converteXmlParaArray($xml->saveXML());
        $correcao = false;
        $dsOperacao = '';
        if($existeTpEvento) $tpEvento = $xml->getElementsByTagName('tpEvento')->item(0)->nodeValue;
        else $tpEvento = '100';
        if($notadestinada) {
            if($tpEvento === '110111') {
                $idTpEvento  = '1';
                $idStatusNfe = '3'; 
                $dsOperacao  = "Nota fiscal cancelada pelo destinatário.";        
                $dtRecbto = $nota["retEvento"]["infEvento"]["dhRegEvento"];
                $dtRecbto = date('d/m/Y H:i:s', strtotime(substr($dtRecbto, 0, 19)));
                $existeNotaDestinada = $this->manifestacaoDestinatarioFactory->existeNotaDestinada($chave);
                
                if(isset($existeNotaDestinada[0])){
                    $this->manifestacaoDestinatarioFactory->atualizaNotaDestinada($idStatusNfe, '', $dtRecbto, $chave);
                }
                else{
                    $nrProxNotadestinada = $this->novoCodigo->getCodeByTableName(self::CONTADOR_NOVOCODIGO_NOTAMANIFESTACAO, $config->nrorg);
                    $this->manifestacaoDestinatarioFactory->incluirNotadestinada($nrProxNotadestinada, '', $chave, '', '',
                                    '', '', '', '', '', '', $idStatusNfe,
                                    '', '', null, null, null, null, null, '', null, '', '', '', '', '', $this->environment->getNrOrgTrab());    
                } 
                $this->quantidadeEventos++;
            }
            else if($tpEvento === '110110') {
                $idTpEvento  = '2';
                $idStatusNfe = '0';
                $dsOperacao  = "Evento de carta de correção registrado para a nota fiscal ".$nota["evento"]["infEvento"]["detEvento"]["xCorrecao"];
                $dtHrEvento  = date('d/m/Y', strtotime(substr($nota["retEvento"]["infEvento"]["dhRegEvento"], 0, 19)));
                $dsCorrecaoNfe = $nota["evento"]["infEvento"]["detEvento"]["xCorrecao"];
                $nrSeqEvento = $this->manifestacaoDestinatarioFactory->getNrSeqEvento($chave)[0]["NRSEQEVENTO"]; 
                $correcao = true;
                $this->manifestacaoDestinatarioFactory->atualizaEventoNotadestinada(null, null, null, 2, $dtHrEvento, $nrSeqEvento, $dsCorrecaoNfe ,$chave);
                $this->quantidadeEventos++;
            }
            else if($tpEvento == '210200' 
                || $tpEvento  == '210210'
                || $tpEvento  == '210220'
                || $tpEvento  == '210240') {
                
                $idTpEvento  = '3';
                $idStatusNfe = '1';
            }
            else {
                $idStatusNfe = '0'; 
                $idTpEvento  = '0';
            }
            
            $sequencial = $this->manifestacaoDestinatarioFactory->getSequencialHistoricoXML($chave)[0]["NRSEQHISTOR"];
            if($sequencial === null) $sequencial = 1;
            else $sequencial += 1;
            $sequencial = str_pad($sequencial, 20, "0", STR_PAD_LEFT);
            
            $notaSincronizada = $this->manifestacaoDestinatarioFactory->getNotaSincronizada($chave);
            
            $cdFilial   = isset($notaSincronizada[0]) ? $notaSincronizada[0]["CDFILIAL"] : '';
            $nrLanctonf = isset($notaSincronizada[0]) ? $notaSincronizada[0]["NRLANCTONF"] : '';
            $idStatus   = isset($notaSincronizada[0]) ? 'S' : 'N';
            
            if($correcao === true) {
                $nProt      = $nota["retEvento"]["infEvento"]["nProt"];
            }
            else {
                try {
                    $nProt      = $nota["evento"]["infEvento"]["detEvento"]["nProt"];
                }
                catch(\Exception $e) {
                    $nProt      = $nota["retEvento"]["infEvento"]["nProt"];
                }
            }
            
            $tipoOper   = isset($this->manifestacaoDestinatarioFactory->buscaHistoricoXML($chave)[0]) ? 1 : 2;
            
            $this->manifestacaoDestinatarioFactory->insereHistoricoXML($chave, $sequencial, date('d/m/Y H:i:s'), $tipoOper, $idStatus, $nProt, $dsOperacao, $cdFilial, $nrLanctonf, $this->environment->getCdOperador(), $this->environment->getNrOrg());
        }
        // TODO 
        // Buscar nota destinada por chave de acesso caso não exista no sistema
    }

    public function manifestarNfe($notasfiscais, $config, $evento = '', $justificativa = '')  {
        $config = json_encode($config);
        $object_config = json_decode($config);
    	if(!isset($this->nfeTools)) {
    		$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, $config);
    		$this->nfeTools = $tools;
        }
        
        $array_erros = array();
        foreach ($notasfiscais as $nota) {
            $erro = $this->manifestaNfe($nota, $evento, $justificativa, $config);
            if($erro) {
                $erro['NRLANCTONF'] = $nota['NRLANCTONF'];
                $erro['NRNOTAFISC'] = $nota['NRNOTAFISC'];
                array_push($array_erros, $erro);
            }
        }
        if(empty($array_erros)) return array("SUCESSO" => "Nota(s) manifestada(s) com sucesso.");
        else return array("ERRO" => $array_erros);
    }

    public function gerarXmlNfeManifestacao($notasfiscais, $config){
        $object_config = json_decode(json_encode($config));
        
    	if(!isset($this->nfeTools)) {
    		$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, json_encode($config));
    		$this->nfeTools = $tools;
        }
        
        $existeNota = false;
        $dir = sys_get_temp_dir();
        $tpAmb = $object_config->tpAmb;

        $zipname = $dir.DIRECTORY_SEPARATOR.uniqid().'XMLs.zip';

        $zip = new \ZipArchive();
        

        if( $zip->open( $zipname , \ZipArchive::CREATE )  === true){
            foreach ($notasfiscais as $nota) {
                $notaDownload = $this->downloadNfe($nota, $dir, $tpAmb);
                
                if($notaDownload) {
                    $existeNota = true;
                    $zip->addFromString('NFe-'.$nota["NRACESSONFE"].'.xml' , file_get_contents($notaDownload));
                    unlink($notaDownload);
                }
            }
            $zip->close();
        }

        if(!$existeNota) {
            return false;
        }
        else {
            return $zipname;
        }
    }
    
    private function buscaNotasSefaz($ultNSU, $numNSU, $cnpj){
        $this->aResposta = array();
        $tpAmb = $this->nfeTools->config->tpAmb;
        $resp = $this->nfeTools->sefazDistDFe($ultNSU, $numNSU, 'AN', $this->aResposta);
        
        $this->aResposta = ReturnNFe::readReturnSefaz('NfeDistribuicaoDFe', $resp);
        
        return $this->aResposta;
    }

    private function manifestaNfe($nota, $tpEvento, $xJust, $config){
        $notaEntidade = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notadestinada')->findOneBy(
                            array('nracessonfe' => $nota['NRACESSONFE']));
        if(($tpEvento == "2" || $tpEvento == "3") && isset($notaEntidade)) {
            return array("CODIGO" => '', "MOTIVO" => "Nota existente no sistema. Não é possivel realizar operações
            do tipo \"Operação não realizada\" ou \"Desconhecimento da operação\".");
        }
        else {
            $this->aResposta = array();
            $chave = $nota['NRACESSONFE'];
            $tpAmb = $this->nfeTools->config->tpAmb;
            
            switch($tpEvento) {
                case "1":
                    $tpEvento = "210200";
                    $dsOperacao    = "Operação Confirmada";
                    break;
                
                case "4": 
                    $idStatusManif = "210210";
                    $dsOperacao    = "Ciência da Emissão";
                    break;
                
                case "2":
                    $idStatusManif = "210220";
                    $dsOperacao    = "Operação Desconhecida";
                    break;
                    
                case "3":
                    $idStatusManif = "210240";
                    $dsOperacao    = "Operação Não Realizada";
                    break;
            }
            
            $eventos    = array("sucesso" => array(), "erro" => array());
            $xmlRetorno = $this->nfeTools->sefazManifesta($chave, $tpEvento, $xJust, 1, $this->aResposta);
            
            $this->aResposta = ReturnNFe::readReturnSefaz('RecepcaoEvento', $xmlRetorno);
            
            return $this->trataLoteRetorno($this->aResposta, $xmlRetorno, $config, $tpEvento);
        }
        
    }

    private function trataLoteRetorno($loteRetorno, $xmlRetorno, $config, $tpEvento) {
        switch($tpEvento) {
            case "210200":
                $idStatusManif = "1";
                $dsOperacao    = "Operação Confirmada";
                break;
            
            case "210210": 
                $idStatusManif = "4";
                $dsOperacao    = "Ciência da Emissão";
                break;
            
            case "210220":
                $idStatusManif = "2";
                $dsOperacao    = "Operação Desconhecida";
                break;
                
            case "210240":
                $idStatusManif = "3";
                $dsOperacao    = "Operação Não Realizada";
                break;
        }

        // Verifica erro no lote
        if($loteRetorno['cStat'] == '128') {
            // Verifica erro nos eventos
            foreach ($loteRetorno['evento'] as $evento) {
                if($evento['cStat'] == '135' || ($evento['cStat'] == '136' && $evento['xMotivo'] == 'Evento registrado, mas nao vinculado a NF-e')) {
                    $notaDestinada = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notadestinada')->findOneBy(
                            array('nracessonfe' => $evento['chNFe']));
                    if($notaDestinada) {
                        $this->manifestacaoDestinatarioFactory->atualizaEventoNotadestinadaManifestada($notaDestinada, $evento, $idStatusManif, $xmlRetorno, $this->nfeTools->config->cdempresa, $this->nfeTools->config->nrorg);
                    }
                    else {
                        $this->manifestacaoDestinatarioFactory->incluirNotadestinadaManifestada($evento, $config, $idStatusManif, $xmlRetorno, $this->nfeTools->config->cdempresa, $this->nfeTools->config->nrorg);
                        $this->manifestacaoDestinatarioFactory->deletaNotaDuplicada($evento["chNFe"], $config->cnpj);
                    }
                    
                    $dadosNf =  $this->manifestacaoDestinatarioFactory->getDadosNf($evento['chNFe']);
                    if(isset($dadosNf[0])) {
                          if($dadosNf[0]["IDGERADAVIAXML"] == "S") {
                              $this->manifestacaoDestinatarioFactory->insereHistoricoXML($chave, $sequencial, date('d/m/Y H:i:s'), $tipoOper, $idStatus, $nProt, $dsOperacao, $cdFilial, $nrLanctonf, $this->environment->getCdOperador(), $this->environment->getNrOrg());
                          }
                    }
                }
                else {                    
                    $this->manifestacaoDestinatarioFactory->atualizaCdStatusManif($evento["chNFe"], $evento["cStat"]);
                    $erro = array($evento['cStat'], $evento['xMotivo'], $tpEvento);
                }
            }
            if(isset($erro)) {
                return array("CODIGO" => $evento['cStat'], "MOTIVO" => $evento['xMotivo']);
            }
        }
        else return array("CODIGO" => $loteRetorno['cStat'], "MOTIVO" => $loteRetorno['xMotivo']);
    }
    
    function formataData($date) {
        $dateDMY  = substr($date, 0, 10);
        $dateHIS  = substr($date, 11, 8);
        $formatedDate = explode('-', $dateDMY);
        $formatedDate = $formatedDate[0].'-'.$formatedDate[1].'-'.$formatedDate[2];
        $formatedDate.= ' '.$dateHIS;
        $formatedDate = strtotime($formatedDate);
        $formatedDate = date("d/m/Y H:i:s", $formatedDate);
        return DateUtil::getDataDeString($formatedDate, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
    }

    private function downloadNfe($nota, $dir, $tpAmb){
        $this->aResposta = array();
        $chave = $nota["NRACESSONFE"];
        $nota = $this->nfeTools->sefazDownload($chave, $tpAmb, $this->aResposta);
        
        $nota = ReturnNFe::readReturnSefaz('NfeDistribuicaoDFe', $nota);

        if(isset($nota)) {
            if(!empty($nota["aDoc"])) {
                file_put_contents($dir.DIRECTORY_SEPARATOR.$chave.'.xml', $nota["aDoc"][0]["doc"]);
                return $dir.DIRECTORY_SEPARATOR.$chave.'.xml';
            }
        }
        else {
            return false;
        }
        
    }

    private function recolocarArrobaEmail($config) {
        $config            = json_decode($config);
        $config->aMailConf = json_decode(str_replace("&#64", "@", json_encode($config->aMailConf)));
        $config            = json_encode($config);
        return $config;
    }
    /**
    *   Converte uma string Xml para um array.
    *    
    *   @param string $stringXml
    *   return  array - Array do XML
    */
    protected function converteXmlParaArray($stringXml) {
        $xml = simplexml_load_string($stringXml);
        $json = json_encode($xml);
        $array = json_decode($json,TRUE);
        return $array;
    }
    
    public function calculaDadosSincronizacao($sincronizacao) {
        $dadosSincronizacao = array();
        $dadosSincronizacao["QTNOTAS"]   = "Foram encontradas ".count($this->notasEncontradas)." nota(s) destinada(s).";
        $dadosSincronizacao["QTEVENTOS"] = "Foram encontrados $this->quantidadeEventos do tipo cancelamento ou Carta de Correção.";
        $dadosSincronizacao["QTINEXIST"] = "Existem ".count($this->notasInexistentes)." notas fiscais destinadas inexistentes no sistema.";
        $dadosSincronizacao["QTIMPORTP"] = "Existem ".count($this->importacaoPendente)." notas fiscais com importação de XML pendente.";
        $dadosSincronizacao["SINCRONIZ"] = "Foram atualizadas $this->sincronizadas notas fiscais existentes no sistema";
        $dadosSincronizacao["NOTASDOWN"] = "Download de $this->notasDownload arquivos XML realizado com sucesso";
        return $dadosSincronizacao;
    }

	public function getToolsNFe($ie, $cnpj, $config) {
	    /* 
	        Função criada para buscar o cnpj vinculada a isncrição estadual, de forma que ele sempre busque o certificado corretp
	    */
	    $cnpjInscEstad = $this->manifestacaoDestinatarioFactory->getCNPJCertificado($ie, json_decode($config)->cdempresa)['NRINSJURIEST'];
	    $certificado = $this->manifestacaoDestinatarioFactory->getCertificado($ie, $cnpjInscEstad);
		
		$certificadoEncontrado = false;
		
		foreach($certificado as $cert) {
		    if($cert["DSDIRCERTIFICADO"]) {
		        $certificadoEncontrado = true;
		        $certify = $cert;
		    }
		}
		
		if($certificadoEncontrado == false) {
		    throw new \Exception("Certificado Digital não encontrado.");
		}
		
		if(empty($certify["CDSENHACERTIF"])){  
		   throw new \Exception("Senha do certificado não informada.");
		}else{
		$senha = $this->decrypt($certify["CDSENHACERTIF"]);
		}
		
		$certificado = $certify["DSDIRCERTIFICADO"];
		
		$certificado = explode('>', $certificado);
        $certificado = file_get_contents($this->fileServerLink.$certificado[1]);
        
        try {
            $certificado = Certificate::readPfx($certificado, $senha);
        }
        catch(\Exception $e) {
            throw new \Exception("Senha do certificado incorreta!");
        }
		
        
        $tools = new NFe($config, $certificado);
			
        return $tools;
        
	}
	
	public function decrypt($text, $salt = 'teknisa') {
	    return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    public function getDefaultFilterManif($row) {
        $cdfilial = ($this->environment->getCdFilial()) ? $this->environment->getCdFilial() : $row["CDFILIAL"];
        $defaultFilter = $this->manifestacaoDestinatarioFactory->getDefaultFilterManif($cdfilial);
        $defaultFilter["DTEMISSAO"] = date("d/m/Y");
        return $defaultFilter;
    }
    
    public function nvl(&$var, $default = ""){
        return isset($var) ? $var
                           : $default;
    }
    
    private function trataRetornoLoteSincronizacao($cnpj, $notas, $object_config, $params, $nsu = '') {
        $cStat = $notas["cStat"];
        $xMotivo = $notas["xMotivo"];
        if($notas["cStat"] == 138) { //Documento localizado.
            
            $ultNSU = $notas["ultNSU"];
            $maxNsu = $notas["maxNSU"];
            if(!$nsu) {
                $this->insereNsuNfe($cnpj, $object_config->tpAmb, $ultNSU, $maxNsu);
            }
            $this->notas = array_merge($this->notas, $notas["aDoc"]);
            
            if(isset($notas["aDoc"])) {
                $this->preencheTabelasSincManifestacao($notas, $object_config);
                $atualizaIdConsultado = true;
                
                foreach($notas["aDoc"] as $notaFiscal) {
                    $nota = $this->converteXMLParaArray($notaFiscal["doc"]);
                    
                    $tipoEvento = false;
                    $tipoRes    = false;
                    $tipoNota   = false;
                    $tipoResEv  = false;
                    $data = $this->manifestacaoDestinatarioFactory->getDataHoraCerta();
                    if($atualizaIdConsultado && $params["FORMAPESQUISA"] == 1) {
                        //Esse processo de voltar o IDCONSULTADO para 'N' será necessário para poder realizar novamente a consulta de todos os NSU's
                        //quando o usuário optar por realizar a consulta dos últimos 3 meses.
                        $this->manifestacaoDestinatarioFactory->atualizaIdConsultado('N', $cnpj, $object_config->tpAmb, $notaFiscal["NSU"]);
                        $atualizaIdConsultado = false;
                    }
                    
                    // ProcEventoNfe
                    if(strtolower(substr($notaFiscal["schema"], 0, 13)) == strtolower('procEventoNFe')) {
                    	$infEvento = $nota["retEvento"]["infEvento"];
                    	$chave =  $infEvento["chNFe"];
                    	
                    	if($infEvento["tpEvento"] == '110111') { //110111: Tipo de Evento de Cancelamento.
                    	    $idStatus = 3;
                    	} 
                    	else {
                    	    $idStatus = 1;
                    	}
                    	
                        $this->atualizaNSUNFe('', $data["DATAHORACERTA"], $infEvento["dhRegEvento"], $chave, $infEvento["cStat"], $infEvento["xMotivo"], $infEvento["xMotivo"], 'S', $infEvento["nProt"], $cnpj, $infEvento["tpAmb"], $notaFiscal["NSU"], $idStatus, $infEvento["tpEvento"], $infEvento["nSeqEvento"], $notaFiscal["doc"]);
                    	$tipoEvento = true;
                    }
                    // ResNFe
                    else if(strtolower(substr($notaFiscal["schema"], 0, 6)) == strtolower('resNFe')) {
                    	$chave = $nota["chNFe"];
                        $this->atualizaNSUNFe('', $data["DATAHORACERTA"], $nota["dhRecbto"], $chave, $cStat, $xMotivo, 'Resumo da NF-e', 'S', $nota["nProt"], $cnpj, $object_config->tpAmb, $notaFiscal["NSU"], $nota["cSitNFe"], '', '', $notaFiscal["doc"]);
                    	$tipoRes = true;
                    	
                    	if(!isset($this->chaves[$chave])) {
            			    $this->chaves[$chave] = $chave; //Apenas notas do tipo "ResNFe" são consideradas para retorno.
            			}
                    }
                    // ProcNFe
                    else if(strtolower(substr($notaFiscal["schema"], 0, 7)) == strtolower('procNFe')) {
                    	$infProt = $nota["protNFe"]["infProt"];
                        $chave = $infProt["chNFe"];
                        $this->atualizaNSUNFe('', $data["DATAHORACERTA"], $infProt["dhRecbto"], $chave, $infProt["cStat"], $infProt["xMotivo"], 'NF-e Processada', 'S', $infProt["nProt"], $cnpj, $infProt["tpAmb"], $notaFiscal["NSU"], '1', '', '', $notaFiscal["doc"]);       
                    	$tipoNota  = true;
                    }
                    // ResEvento
                    else if(strtolower(substr($notaFiscal["schema"], 0, 9)) == strtolower('resEvento')) {
                        $chave = $nota["chNFe"];
                        $dhRecbto = isset($nota["dhRecbto"]) ? $nota["dhRecbto"] : null;
                        $nProt    = isset($nota["nProt"])    ? $nota["nProt"]    : null;
                        $tipoResEv = true;
                        $this->atualizaNSUNFe('', $data["DATAHORACERTA"], $dhRecbto, $chave, $cStat, $xMotivo, 'Resumo do Evento da NF-e', 'S', $nProt, $cnpj, $object_config->tpAmb, $notaFiscal["NSU"], '', $nota["tpEvento"], $nota["nSeqEvento"], $notaFiscal["doc"]);
                    }
        			
                    if($tipoEvento === true || $tipoRes === true) {
                        $notaSincronizada = $this->manifestacaoDestinatarioFactory->getNotaSincronizada($chave);
                        $somenteXML = $this->manifestacaoDestinatarioFactory->somenteXML($chave);
                        
                        if($tipoRes) {
                    	    if(!isset($somenteXML[0]) && !isset($notaSincronizada[0])) {
                			    if(!isset($this->notasInexistentes[$chave])) {
                                    $this->notasInexistentes[$chave] = $chave;
                    	        }
                			}
                    	}
                        if(isset($notaSincronizada[0])) {
                        	$this->manifestacaoDestinatarioFactory->relacionarNF($notaSincronizada[0]["CDFILIAL"], $notaSincronizada[0]["NRLANCTONF"], $notaSincronizada[0]["CDINSCESTA"], $chave);
                        	$this->sincronizadas++;
                        	
                        	if(isset($somenteXML[0])) {
                                $this->manifestacaoDestinatarioFactory->intg($chave, $chave);
                        	}
                        }
                        else {
                        	if(isset($somenteXML[0])) {
                            	if(!isset($this->importacaoPendente[$chave])) {
                                    $this->importacaoPendente[$chave] = $chave;
                                }
                            	$this->manifestacaoDestinatarioFactory->intg($chave, $chave);
                        	}
                        }
                    }
                    else {
                    	$this->xmlsNota[$chave] = $notaFiscal["doc"];
                    }
                    
                }
            }
        }
        else if($notas["cStat"] == 137) { // Nenhum documento localizado.
            $data = $this->manifestacaoDestinatarioFactory->getDataHoraCerta();
            if($nsu) {
                $this->atualizaNSUNFe('', $data["DATAHORACERTA"], $notas["dhResp"], '', $cStat, $xMotivo, '', 'S', '', $cnpj, $object_config->tpAmb, $nsu, '', '', '', '');
            }
            else {
                $ultNSU = $notas["ultNSU"];
                $maxNsu = $notas["maxNSU"];
                $this->insereNsuNfe($cnpj, $object_config->tpAmb, $ultNSU, $maxNsu);
            }
            
            throw new \Exception("Nenhum documento localizado.");
        }
        else if($cStat == 656) throw new \Exception($xMotivo, $cStat); //Consumo indevido.
    }
    
    private function insereNsuNfe($cnpj, $tpAmb, $ultNsu, $maxNsu){
        $maxNrNsu = $this->manifestacaoDestinatarioFactory->maxNrNsu($cnpj, $tpAmb, 'T')["MAXNRNSU"];
        $data = $this->manifestacaoDestinatarioFactory->getDataHoraCerta();

        if($maxNrNsu == 0) {
            for($i = ($ultNsu - 100); $i <= $maxNsu; $i++) {
                if($i >= 0) {
                    $nsu = str_pad($i, 15, "0", STR_PAD_LEFT);
                    $nsuExiste = $this->manifestacaoDestinatarioFactory->consultaNsu($cnpj, $tpAmb, $nsu);
    
                    if(empty($nsuExiste)) {
                        $this->manifestacaoDestinatarioFactory->insereNsuNfe($cnpj, $tpAmb, $nsu, $data["DATAHORACERTA"], $data["DATAHORACERTA"], '', '', '', '', 'N', '', '', '', '', '', $this->environment->getNrOrgTrab());
                    }
                }
            }
        }
        else {
            for($i = $maxNrNsu; $i <= $maxNsu; $i++) {
                $nsu = str_pad($i, 15, "0", STR_PAD_LEFT);
                $nsuExiste = $this->manifestacaoDestinatarioFactory->consultaNsu($cnpj, $tpAmb, $nsu);
                if(empty($nsuExiste)) {
                    $this->manifestacaoDestinatarioFactory->insereNsuNfe($cnpj, $tpAmb, $nsu, $data["DATAHORACERTA"], $data["DATAHORACERTA"], '', '', '', '', 'N', '', '', '', '', '', $this->environment->getNrOrgTrab());
                }
            }
        }

    }
    
    private function atualizaNsuNfe($dtinsnrnsu, $dtupdnrnsu, $dtrecebto, $nracessonfe, $cdstatus, $dsmotivo, $dsservico, $idconsultado, $nrprotocolo, $nrinsjuriest, $tpamb, $nrnsu, $idstatusnfe, $cdservico, $nrseqservico, $xml) {
        $dtrecebto = date('d/m/Y H:i:s', strtotime(substr($dtrecebto, 0, 19)));
        $this->manifestacaoDestinatarioFactory->atualizaNsuNfe($dtinsnrnsu, $dtupdnrnsu, $nracessonfe, $cdstatus, $dsmotivo, $dsservico, $idconsultado, $dtrecebto, $nrprotocolo, $idstatusnfe, $cdservico, $nrseqservico, $nrinsjuriest, $tpamb, $nrnsu);
        
        if($xml) {
            if(!$this->manifestacaoDestinatarioFactory->existeXmlNsu($nrinsjuriest, $tpamb, $nrnsu)) {
                $this->manifestacaoDestinatarioFactory->insereXmlNsu($nrinsjuriest, $tpamb, $nrnsu, $xml, $this->environment->getNrOrgTrab());
            }
        }
    }
    
    private function atualizaNSUFaltante($nrinsjuriest, $tpAmb, $object_config, $params, $dtRecbtoIni, $dtRecbtoFin) {
        $nrLista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrOrgTrab(), 1, 10);
        $i = 0;
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'INSERE_NSU_FALTANTE');
        $procedure->addParam(new Param('P_NRLISTA',         Param::PARAM_INPUT, $nrLista,                           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURIEST',    Param::PARAM_INPUT, $nrinsjuriest,                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDAMBIENTENFE',   Param::PARAM_INPUT, $tpAmb,                             Param::PARAM_TYPE_STR));
        $procedure->execute();
        
        $lista = $this->manifestacaoDestinatarioFactory->getCodLista($nrLista);
        foreach($lista as $nsu) {
            $i++;
            if($i == 21) break;
            
            try {
                $nsuLista = (int) $nsu["CDLISTA"];
                $this->buscaNotasSefaz($nsuLista, $nsuLista, $nrinsjuriest);
                $notas = $this->aResposta;
                $this->trataRetornoLoteSincronizacao($nrinsjuriest, $notas, $object_config, $params, $nsu["CDLISTA"]);
            }
            catch(\Exception $e) {
                if($e->getMessage() != "Nenhum documento localizado.") $this->erros[] = $e->getMessage() . ' - '. $e->getCode();
                if($e->getCode() == 656) break;  // Rejeição 656: Consumo Indevido.
            }
        }
        $this->manifestacaoDestinatarioFactory->deletaLista($nrLista);
    }
    
    public function setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $newFileUploader) {
        if($useVpd){
            $this->entityManager = $newEntityManager;
            $this->novoCodigo = new NewCode($newEntityManager->getConnection(), 'NOVO_CODIGO_SC');
        }
        
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
        $this->manifestacaoDestinatarioFactory->setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $newFileUploader);
    }
    
    public function getManifestacaoFactory() {
        return $this->manifestacaoDestinatarioFactory;
    }
}
