<?php
namespace Teknisa\DF\Modules\Saida\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Doctrine\ORM\EntityManager;
use Util\Environment;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;
use Teknisa\Model\Financeiro\Entities\Contareceber;
use Teknisa\DF\Modules\Saida\NFSe\Utilities\Util;
use Teknisa\DF\Modules\Saida\Services\Dfs65001ParametrosNfseInscmuniService;
use Teknisa\DF\Modules\Saida\Services\Dfs61000ConsultaloteService;
use Teknisa\DF\Modules\Saida\Controllers\BirtReportController;
use Teknisa\DF\Services\AprovacaoLancamentosService;
use Teknisa\DF\Services\EmissaoBoletoService;
use Teknisa\DF\Controllers\EmissaoBoletoController;
use \Teknisa\Libs\Util\SMTP;

class Dfs61000ConsultaloteController extends \Teknisa\DF\Controllers\TeknisaCrud {
    const CAMINHO_MUNICIPIOS = 
        array("4123" => "Teknisa\DF\Modules\Saida\NFSe\NFSeBeloHorizonte",
              "4371" => "Teknisa\DF\Modules\Saida\NFSe\NFSeContagem",
              "4895" => "Teknisa\DF\Modules\Saida\NFSe\NFSeNovaLima",
              "6001" => "Teknisa\DF\Modules\Saida\NFSe\NFSeRioDeJaneiro",
              "6291" => "Teknisa\DF\Modules\Saida\NFSe\NFSeCampinas",
              "7777" => "Teknisa\DF\Modules\Saida\NFSe\NFSePontaGrossa",
              "9067" => "Teknisa\DF\Modules\Saida\NFSe\NFSeCuiaba",
              "7107" => "Teknisa\DF\Modules\Saida\NFSe\NFSeSaoPaulo",
              "3849" => "Teknisa\DF\Modules\Saida\NFSe\NFSeSalvador"
            );
    protected $dataSourceName = 'loterps_numerorps';
    protected $environment;
    protected $connection;
    protected $entityManager;
    protected $newCode;
    protected $paramNfseService;
    protected $birtController;
    protected $smtpService;
    protected $certUploadDirectory;
    protected $xmlDirectory;
    protected $aprovacaoLancamentosService;
    protected $emissaoBoletoService;
    protected $emissaoBoletoController;
    protected $errosAprovacao = false;

    public function __construct($crudService, $parameterBag, $environment, EntityManager $entityManager, $newCode, Dfs65001ParametrosNfseInscmuniService $paramNfseService, BirtReportController $birtController, SMTP $smtpService, $certUploadDir, $xmlDir, AprovacaoLancamentosService $aprovacaoLancamentosService = null, EmissaoBoletoService $emissaoBoletoService = null, EmissaoBoletoController $emissaoBoletoController, Dfs61000ConsultaloteService $consultaLoteService, $fileServerURL) {
        parent::__construct($crudService, $parameterBag);
        $this->environment = $environment;
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->paramNfseService = $paramNfseService;
        $this->birtController = $birtController;
        $this->smtpService = $smtpService;
        $this->certUploadDirectory = $certUploadDir;
        $this->xmlDirectory = $xmlDir;
        $this->aprovacaoLancamentosService = $aprovacaoLancamentosService;
        $this->emissaoBoletoService = $emissaoBoletoService;
        $this->emissaoBoletoController = $emissaoBoletoController;
        $this->consultaLoteService = $consultaLoteService;
        $this->fileServerBaseUrl = $fileServerURL;
    }

    public function consultarLotes(Request\DataSet $request, Response $response) {
        try {
            $request = $request->getDataSet()->getRows();
            $cdempresa = $request[0]["CDEMPLOTERPS"];
            $cdinscmuni = $request[0]["CDINSCMUNI"];
            $nrorg = $this->environment->getNrOrg() ? $this->environment->getNrOrg() :  $request[0]["NRORG"];
            $lotesErro = array();
            $dataset = Util::getDadosGeracao($this->connection, $cdempresa, $cdinscmuni, $nrorg);

            if (!is_null($dataset)) {
                if(array_key_exists($dataset["CDMUNICIPIO"], self::CAMINHO_MUNICIPIOS)) {
                    $caminho = self::CAMINHO_MUNICIPIOS[$dataset["CDMUNICIPIO"]];
                    $lotesComErro = 0;
                    $msg = "";
                    $funcionou = true;
                    $consultarNovamente = false;
                    $lotesAceitos = array();
        
                    foreach ($request as $lote){
                        if($caminho !== 'Teknisa\DF\Modules\Saida\NFSe\NFSeNovaLima') {
                        $localServidor = $this->getLocalServidor($cdempresa, $cdinscmuni, $nrorg);
                            if ($localServidor == 1) { 
                                $nfse = new $caminho($this->connection, $this->environment, $this->newCode, $this->xmlDirectory, $this->certUploadDirectory);
                            } 
                            else if ($localServidor == 2) {
                                $arrayCertificado = $this->getLinkCertificado($cdempresa, $cdinscmuni, $nrorg);
                                $nfse = new $caminho($this->connection, $this->environment, $this->newCode, $this->xmlDirectory, null, $arrayCertificado["CERTIFICADO"], $arrayCertificado['CERTKEY']);
                            }
                        }
                        else {
                            $nfse = new $caminho($this->connection, $this->environment, $this->newCode, $this->xmlDirectory);
                        }
                        
                        $retorno = $nfse->consultarLoteRps($lote);
                        
                        if (is_array($retorno)) {
                            $lotesComErro++;
                            foreach ($retorno as $lote) {
                                if(isset($lote["RPS"])) {
                                    // Verifica se o erro não é de lote não encontrado para Campinas
                                    if($dataset["CDMUNICIPIO"] !== '6291' || ($dataset["CDMUNICIPIO"] == '6291' && $lote["CODIGO"] !== '1105'))  {
                                        $notaErro = $this->consultaLoteService->getErroNotaPorRps($lote["RPS"],  $nrorg, $cdempresa, $lote['DESCRICAO']); 
                                        if(!is_null($notaErro)) {
                                            array_push($lotesErro, $notaErro);
                                        }
                                        else {
                                            array_push($lotesErro, array(
                                                "NRLANCTONF" => '',
                                                "NRNOTAFISC" => '',
                                                "TIPODEERRO" => '3',
                                                "VRNOTAFISC" => '',
                                                "DTEMISSAO"  => '',
                                                "DSERRO"     => $lote["DESCRICAO"]
                                            ));
                                        }
                                        
                                    }
                                    else {
                                         array_push($lotesErro, array("NRLANCTONF" => "",
                                                                "NRNOTAFISC" => "",
                                                                "TIPODEERRO" => '3',
                                                                "VRNOTAFISC" => "",
                                                                "DTEMISSAO"  => "",
                                                                "DSERRO"     => $lote["DESCRICAO"]));
                                    }
                                }
                                else {
                                     array_push($lotesErro, array("NRLANCTONF" => "",
                                                            "NRNOTAFISC" => "",
                                                            "TIPODEERRO" => '3',
                                                            "VRNOTAFISC" => "",
                                                            "DTEMISSAO"  => "",
                                                            "DSERRO"     => $lote["DESCRICAO"]));
                                }

                                $msg .= $lote["CODIGO"] . " - " . str_replace(">", "\"", str_replace("<", "\"", $lote["DESCRICAO"])) . "\n";
                            }
                        }
                        else if ($retorno === null) {
                            $consultarNovamente = true;
                            break;
                        }
                        else if ($retorno === false) {
                            $funcionou = false;
                            break;
                        }
                        else {
                            $lotesAceitos[] = $lote;
                        }
                    }
        
                    if ($consultarNovamente) {
                        $response->addMessage(new Response\Message("Lote ainda não foi processado. Tente novamente mais tarde."));
                    }
                    else {
                        if ($funcionou === true) {
                            if ($lotesComErro > 0) {
                                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lotesErro', $lotesErro));
                            }
                            $lotesSucesso = count($request) - $lotesComErro;
                            if($lotesSucesso > 0) $response->addMessage(new Response\Message(count($request) - $lotesComErro . " lote(s) atualizado(s) com sucesso!"));
                            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lotesAceitos', $lotesAceitos));
                        }
                        else {
                            $response->addMessage(new Response\Message("Falha na consulta de lote."));
                        }
                    }
                }
                else  $response->addMessage(new Response\Message("Funções de NFS-e não habilitada para esse município."));
            }
            else {
                $response->addMessage(new Response\Message("Funções de NFS-e não habilitada para esse município."));
            }
        }
		catch (\Exception $e) {
			$response->setError(new Response\Error($e->getMessage(), $e->getCode()));
		}
    }
    

    public function enviarEspelhoNfse(Request\DataSet $request, Response $response) {
        try {
            $lotes = $request->getDataSet()->getRows();
            $nrorg        = $lotes[0]["NRORG"];
            $cdempresa    = $lotes[0]["CDEMPLOTERPS"];
            if(Util::utilizaEnvioEmail($this->connection, $cdempresa, $nrorg) === 'S' 
                && Util::utilizaAprovacaoAutomatica($this->connection, $cdempresa, $nrorg) === 'S') {
                if(isset($lotes[0]['notaserro'])) {
                    $notasErro = $lotes[0]['notaserro'];
                    unset($lotes[0]['notaserro']);
                }
                $count = 0;
                foreach ($lotes as $lote) {
                    $nrorg        = $lote["NRORG"];
                    $cdempresa    = $lote["CDEMPLOTERPS"];
                    $cdinscmuni   = $lote["CDINSCMUNI"];
                    $cdinscesta   = $lote["CDINSCESTA"];
                    $nrinsjuriest = $lote["NRINSJURIEST"];
                    $nrseqloterps = $lote["NRSEQLOTERPS"];
                    $nomeRelatorioDanfe = $this->paramNfseService->getModeloNfse($cdempresa, $cdinscmuni, $nrorg);
                    $notas = $this->getNotasLote($cdempresa, $cdinscmuni, $cdinscesta, $nrinsjuriest, $nrseqloterps);
                    if (is_array($notas) && count($notas) > 0) {
                        $emailEmpresa = $this->retornaEmailEmpresa($notas[0]);
                        $cdempresa =  $notas[0]["CDEMPRESA"];
                        $notasRestantes = $notas;
                        foreach ($notas as $nota) {
                            $naoEnvia = false;
                            if(isset($notasErro[0])) {
                                foreach($notasErro as $notaErro) {
                                    if($nota['NRNOTAFISC'] === $notaErro) $naoEnvia = true;
                                }
                            } 
                            if($naoEnvia === false) {
                                $this->enviarNota($request, $response, $emailEmpresa, $cdempresa, $nota, $count, $nomeRelatorioDanfe, $nrinsjuriest, $nrorg);
                            }
                            unset($notasRestantes[$count]);
                            $count ++;
                        }
                    }
                }
            }
        }
		catch (\Exception $e) {
	        $messageError = $e->getMessage();
		    if($messageError === 'Ocorreu um erro ao enviar o e-mail. Contate o administrador do sistema.') {
    	        $messageError.= '. A(s) nota(s) ';
    	        $first = true;
    	        $notas = [];
    	        
    		    foreach ($notasRestantes as $nota) {
    		        if(!$first)array_push($notas, $nota['NRNOTAFISC']);
    		        else {
    		           array_push($notas, $nota['NRNOTAFISC']);
    		           $first = false;
    		        }
    		    }
		    }
			//$response->setError(new Response\Error($notas, 500));
		}
    }
    
    //Envia email após a consulta do lote
    public function enviarEmailAutomatico(Request\DataSet $request, Response $response) {
        $notas = $request->getDataSet()->getRows();
          try {
            $count = 0;
            foreach ($notas as $nota) { 
                $notaAtualizada = $this->consultaLoteService->getNotaAtualizada($nota)[0];
                
                $nrseqloterps   = $notaAtualizada['NRSEQLOTERPS'];
                $cdemploterps   = $notaAtualizada['CDEMPLOTERPS']; 
                $nrorg          = $notaAtualizada['NRORG'];
                $cdempresa      = $notaAtualizada['CDEMPRESA'];
                
                // Verifica se a nota foi transmitida
                if($notaAtualizada['IDTRANSMISRPS'] === '4' && $notaAtualizada['NRSEQLOTERPS']){
                    // Verifica se pode enviar o email de forma automática
                    if (Util::utilizaEnvioEmail($this->connection, $cdempresa, $nrorg) === 'S') {
                         
                        $lote = $this->consultaLoteService->getLote($nrseqloterps, $cdemploterps, $nrorg);
                       
                        if(isset($lote[0])) {
                            $lote = $lote[0];
                            $nrorg        = $lote["NRORG"];
                            $cdempresa    = $lote["CDEMPLOTERPS"];
                            $cdinscmuni   = $lote["CDINSCMUNI"];
                            $nrinsjuriest = $lote["NRINSJURIEST"];
                            
                            $nomeRelatorioDanfe = $this->paramNfseService->getModeloNfse($cdempresa, $cdinscmuni, $nrorg);
                            
                            $emailEmpresa = $this->retornaEmailEmpresa($notaAtualizada);
                        
                            $this->enviarNota($request, $response, $emailEmpresa, $cdempresa, $notaAtualizada, $count, $nomeRelatorioDanfe, $nrinsjuriest, $nrorg);
                            
                            $response->addMessage(new Response\Message("Email enviado com sucesso"));
                        }
                        else {
                            throw new \Exception("Lote não encontrado.");
                            $response->setError(new Response\Error('lote não encontrado'));
                        }
                    } else {
                        throw new \Exception("O envio de email automático não está autorizado.");
                        $response->setError(new Response\Error("Erro. O envio de email automático não está autorizado"));
                    }
                    
                } else {
                        throw new \Exception("A nota não foi processada.");
                        $response->setError(new Response\Error("Erro. A nota não foi processada"));
                }
                $count ++;
            }
        } 
        catch(\Exception $e) {
            $errorMessage = 'Nota de lançamento '.$notaAtualizada['NRLANCTONF']. ': '.$e->getMessage();
            $response->setError(new Response\Error($errorMessage, $e->getCode()));
        }
    }
    
    public function enviarEspelhoNfseNota(Request\DataSet $request, Response $response) {
        $notas = $request->getDataSet()->getRows();
        $notasNaoAprovadas = array();
        try {
            foreach ($notas as $nota) {
                if($nota['IDTRANSMISRPS'] != '4' || !$nota['NRSEQLOTERPS']) throw new \Exception("Uma ou mais notas selecionadas não foram processadas ou foram processadas com erro");
            }
            $count = 0;
            $cdempresa = $notas[0]['CDEMPRESA'];
            $nrorg     = $notas[0]['NRORG'];
            foreach($notas as $nota) {
                if($nota['IDSTATUS'] == 'P') {
                    if(Util::utilizaAprovacaoAutomatica($this->connection, $cdempresa, $nrorg) === 'S'){
                        $errosAprovacao = false;
                        $errosAprovacao = $this->aprovacaoLancamentosService->aprovar($nota['NRORG'], $nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $response);
                        if(!$errosAprovacao) $nota['IDSTATUS'] = 'A';
                        else {
                            $errosAprovacao[0]['NRLANCTONF'] = $nota['NRLANCTONF'];
                            $errosAprovacao[0]['TIPODEERRO'] = '1';
                            array_push($notasNaoAprovadas, $errosAprovacao[0]);
                        }
                    } else throw new \Exception("Não foi possivel realizar a aprovação automática. Realize a aprovação manual e tente novamente.");
                }
                if($nota['IDSTATUS'] == 'A' || Util::utilizaEnvioEmail($this->connection, $cdempresa, $nrorg) === 'S') {
                    $nrseqloterps = $nota['NRSEQLOTERPS'];
                    $cdemploterps = $nota['CDEMPLOTERPS'];
                    $nrorg        = $nota['NRORG'];
                    
                    $lote = $this->consultaLoteService->getLote($nrseqloterps, $cdemploterps, $nrorg);
                    
                    if(isset($lote[0])) {
                        $lote = $lote[0];
                        $nrorg        = $lote["NRORG"];
                        $cdempresa    = $lote["CDEMPLOTERPS"];
                        $cdinscmuni   = $lote["CDINSCMUNI"];
                        $nrinsjuriest = $lote["NRINSJURIEST"];
                        
                        $nomeRelatorioDanfe = $this->paramNfseService->getModeloNfse($cdempresa, $cdinscmuni, $nrorg);
                        
                        $emailEmpresa = $this->retornaEmailEmpresa($nota);
                        
                        $this->enviarNota($request, $response, $emailEmpresa, $cdempresa, $nota, $count, $nomeRelatorioDanfe, $nrinsjuriest, $nrorg);
                    }
                    else throw new \Exception("Lote não encontrado");
                }
                $count ++;
            }
        }
        catch(\Exception $e) {
            $errorMessage = 'Nota '.$nota['NRNOTAFISC']. ': '.$e->getMessage();
            if($e->getMessage() !== 'Uma ou mais notas selecionadas não foram processadas ou foram processadas com erro' 
                    && $e->getMessage() !== 'Não foi possivel realizar a aprovação automática. Realize a aprovação manual e tente novamente') {
                $errorMessage = 'Nota '.$nota['NRNOTAFISC']. ': '.$e->getMessage();
            }
            else $nota['NRNOTAFISC'].': '.$errorMessage = $e->getMessage();
            $response->setError(new Response\Error($errorMessage, $e->getCode()));
        }
        if(isset($errosAprovacao[0])) {
            $response->setError(new Response\Error($notasNaoAprovadas, 500));
        }
    }
    
    private function enviarNota($request, $response, $emailEmpresa, $cdempresa, $nota, $count, $nomeRelatorioDanfe, $nrinsjuriest, $nrorg) {
        try {
            $cdfilial             = $nota["CDFILIAL"];
            $nrlanctonf           = $nota["NRLANCTONF"];
            $nrnotafisc           = $nota["NRNOTAFISC"];
            $cdverifnfse          = $nota["CDVERIFNFSE"];
            $razaoSocialPrestador = $nota["NMRAZSOCEMPR"];
            $inscricaoTomador     = $nota["NRINSJURCLIE"];
            $razaoSocialTomador   = $nota["NMRAZSOCCLIE"];
            $dataEmissao          = $nota["DTEMISSAO"];
            $cdcliente            = $nota["CDCLIENTE"];
            $idstatus             = $nota["IDSTATUS"]; //adicionando idstatus para ser passado la em baixo
    
            $clienteRecebeBoleto  = Util::clienteRecebeBoleto($this->connection, $cdcliente);
            if($count === 0) {
                $indexNota = 0;
                $indexBoleto = 1;
                $count = 1;
            }
            else {
                $numeroMessages = count($response->getMessages());
                $indexNota = $numeroMessages;
                $indexBoleto = $numeroMessages + 1;
            }
            $emailCliente = Util::getEmailCliente($this->connection, $cdfilial, $nrlanctonf, $this->environment->getNrOrg());
    
            // $emailCliente = "usuario@teknisa.com"; // Teste - email que quer receber o conteúdo
    
            $caminhoDanfe = $this->gerarDanfe($nomeRelatorioDanfe, $cdfilial, $nrlanctonf, $nrnotafisc, $request, $response, $indexNota, $this->environment->getNrOrg());

            if($clienteRecebeBoleto === 'S') {
                $caminhoBoletos = $this->gerarBoletos($this->environment->getNrOrg(), $cdfilial, $nrlanctonf, $nrnotafisc, $request, $response, $indexBoleto);
                $enviaBoleto = isset($response->getMessages()[$indexBoleto]);
            }
            else {
                $caminhoBoletos = false;
                $enviaBoleto = false;
            }

            $emailEnviado = true;

            if (!empty($emailCliente) &&
                (!empty($caminhoDanfe)  && ($clienteRecebeBoleto == 'N' || !empty($caminhoBoletos)) && ($enviaBoleto || $clienteRecebeBoleto == 'N'))) {
                    $emailEnviado = $this->enviarEmail($cdempresa, $emailEmpresa, $emailCliente, $nrnotafisc, $cdverifnfse, $nrinsjuriest, $razaoSocialPrestador, $inscricaoTomador, $razaoSocialTomador, $dataEmissao, $caminhoDanfe, $caminhoBoletos, $enviaBoleto, $idstatus); // Passar o idstatus de aprovação da nota para ser conferido lá em baixo
                    if($emailEnviado) $this->alteraStatusEmail($nrlanctonf, $cdfilial, $nrorg);
            }
        }
        catch(\Exception $e) {
            throw $e;
        }
        finally {
            if(!isset($caminhoDanfe) || !($emailEnviado)) throw new \Exception('Ocorreu um erro ao enviar o e-mail. Contate o administrador do sistema.');
            $this->unlinkAnexos($caminhoDanfe, $caminhoBoletos);
        }
    }

    private function unlinkAnexos($caminhoDanfe, $caminhoBoletos){
        if($caminhoDanfe && is_file($caminhoDanfe))
            unlink($caminhoDanfe);
        if($caminhoBoletos != false) {
            foreach($caminhoBoletos as $boleto) {
                if(is_file($boleto))
                    unlink($boleto);
            }
        }
    }

    private function gerarBoletos($nrorg, $cdfilial, $nrlanctonf, $nrnotafisc, $request, $response, $indexBoleto){
        $contareceber = $this->entityManager->getRepository('Teknisa\Model\Financeiro\Entities\Contareceber')->findBy(
                array('cdfilial' => $cdfilial, 'nrlanctonf' => $nrlanctonf));
        $caminhoBoletos = array();
        
        if($contareceber){
            foreach($contareceber as $conta){
                if($conta->getNrlanbolrec()){
                    $nomeRelatorioBoleto = $this->emissaoBoletoService->emiteBoleto($nrorg, $conta->getCdfilibolrec(), $conta->getNrlanbolrec());
                    if (!empty($nomeRelatorioBoleto)) {
                        $rowBirt = new Row();
                        $rowBirt->set("__report_name", $nomeRelatorioBoleto);
                        $rowBirt->set("CDFILIAL", $conta->getCdfilibolrec());
                        $rowBirt->set("NRLANBOLREC", $conta->getNrlanbolrec());
                        $rowBirt->set("NRORG", $nrorg);
                        $rowBirt->set("MOD_PATH", 'df');
                        $birt = new Request\Row($rowBirt, "POST", "/createRemoteReport", $request->getUserId());
                        $this->birtController->createRemoteReport($birt, $response);
                    }
                }
            }
            if (isset($response->getMessages()[$indexBoleto])) {
                $_mensagens = $response->getMessages();
                $count = 0;
                foreach($_mensagens as $key => $message){
                    if($count >= $indexBoleto) {
                        $nBoleto = ($count - $indexBoleto) + 1;
                        $urlRelatorio = $message->getMessage();
                        $caminhoBoletos[] = $this->salvarRelatorio($nrnotafisc . '_boleto_' . $nBoleto, $urlRelatorio);
                    }
                    $count++;
                }
            }
        }

        return $caminhoBoletos;
    }
    
    private static function removerDanfeMensagens($mensagens, $contareceber){
        if(count($mensagens) > count($contareceber))
            array_shift($mensagens);
        return $mensagens;
    }

    private function gerarDanfe($nomeRelatorioDanfe, $cdfilial, $nrlanctonf, $nrnotafisc, $request, $response, $indexNota, $nrorg){
        if (!empty($nomeRelatorioDanfe)) {
            $rowBirt = new Row();
            $rowBirt->set("__report_name", $nomeRelatorioDanfe);
            $rowBirt->set("CDFILIAL", $cdfilial);
            $rowBirt->set("NRLANCTONF", $nrlanctonf);
            $rowBirt->set("NRORG", $nrorg);
            $requestBirt = new Request\Row($rowBirt, "POST", "/createRemoteReport", $request->getUserId(), 'aaa');
            $this->birtController->createRemoteReport($requestBirt, $response);
            // var_dump([$response]);
            if (isset($response->getMessages()[$indexNota])) {
                $urlRelatorio = $response->getMessages()[$indexNota]->getMessage();
                return $this->salvarRelatorio($nrnotafisc, $urlRelatorio);
            }
        }
    }

    public function enviarEmail($cdempresa, $remetente, $destinatarios, $nrnotafisc, $cdverifnfse, $inscricaoPrestador, $razaoSocialPrestador, $inscricaoTomador, $razaoSocialTomador, $dataEmissao, $caminhoDanfe, $caminhoBoletos, $enviaBoleto, $idstatus) {
        $dataEmissao = explode(" ", $dataEmissao);
        $dataEmissao = $dataEmissao[0];
        if($remetente == 'Faturamento@teknisa.com' || $remetente == 'faturamento@teknisa.com') $fromname = 'Faturamento Teknisa';
        else $fromname = $remetente;
        $destinatarios = str_replace(";", ",", $destinatarios);
        $destinatarios = str_replace(" ", "", $destinatarios);
        $destinatarios = explode(",", $destinatarios);

        $inscricaoPrestador = self::aplicarMascaraNumerica("##.###.###/####-##", $inscricaoPrestador);

        if (strlen($inscricaoTomador) == 14)
            $inscricaoTomador = self::aplicarMascaraNumerica("##.###.###/####-##", $inscricaoTomador);
        else if (strlen($inscricaoTomador) == 11)
            $inscricaoTomador = self::aplicarMascaraNumerica("###.###.###-##", $inscricaoTomador);

        $informacoesEmail = Util::getInformacoesEmailEmpresa($this->connection, $this->environment->getNrOrg(), $cdempresa );
        
        $assunto          = $this->retornaAssuntoEmail($informacoesEmail[0]['DSASSUNTONFE'], $nrnotafisc, $dataEmissao, $razaoSocialPrestador);
        $logoEmpresa = '';
        
        if(!$informacoesEmail[0]['DSMENSEMAILNFE']) {
            $logo = $this->getLogoEmpresa($cdempresa)[0]['DSDIRLOGOMARCA'];
            if($logo != null) {
                $logoEmpresa = 'http://midiatest.teknisa.com/files/'.$this->getLogoEmpresa($cdempresa)[0]['DSDIRLOGOMARCA'];
            }
            else $logoEmpresa = '';
        }
        $cdPrefeitura     = $this->getPrefeitura($cdempresa)[0]['CDMUNICIPIO'];
        $prefeitura       = $this->getPrefeitura($cdempresa)[0]['NMMUNICIPIO'];
        $corpo            = $this->retornaCorpoEmail($informacoesEmail[0]['DSMENSEMAILNFE'], $nrnotafisc, $cdverifnfse, $razaoSocialPrestador, $razaoSocialTomador, $dataEmissao, $inscricaoPrestador, $inscricaoTomador, $logoEmpresa, $cdPrefeitura, $prefeitura);
        $this->smtpService->setFrom($remetente);
        $this->smtpService->setFromName($fromname);
        if($caminhoDanfe === $caminhoBoletos) $caminhoBoletos = null;
        if(!empty($caminhoDanfe))
            $this->smtpService->addAttachment($caminhoDanfe);
        if(!empty($caminhoBoletos) && $enviaBoleto && $idstatus === 'A'){ // Devo garantir que a nota foi aprovada para anexar tb o boleto
            foreach($caminhoBoletos as $boleto){
                $this->smtpService->addAttachment($boleto);
            }
        }
        $this->smtpService->setSubject($assunto);
        
        $enviado  = $this->smtpService->sendMail($corpo, $destinatarios, '', $informacoesEmail[0]['DSEMAILCCNFE']);
        
        return $enviado;
    }

    public function salvarRelatorio($nomeArquivo, $reportUrl) {
        try {
            $cURLRequest = new cURLRequest($reportUrl, 'GET');
            $report = $cURLRequest->request('');
            $requestInfo = $cURLRequest->getRequestInfo();
            if ($requestInfo['http_code'] !== HTTPInterface::STATUS_CODE_OK) {
                throw new \Exception('Servidor de emissão de relatórios indisponível. Contate o administrador de rede');
            }
        }
        catch (\Exception $e) {
            throw $e;
        }
        
        $reportPath = sys_get_temp_dir().DIRECTORY_SEPARATOR."NFSe_$nomeArquivo.pdf";
      
        file_put_contents($reportPath, $report);
        unset($report);
        unset($cURLRequest);
        return $reportPath;
    }
    
    public function getPrefeitura($cdempresa) {
        $sql = "SELECT NVL(M.CDMUNICSIAFI, M.CDMUNICIPIO) AS CDMUNICIPIO,
                  NMMUNICIPIO
                FROM EMPRESA E,
                  MUNICIPIO M
                WHERE E.CDMUNICIPIO = M.CDMUNICIPIO
                AND CDEMPRESA       = :P_CDEMPRESA";
        $params = array(
            "P_CDEMPRESA"   => $cdempresa
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getLogoEmpresa($cdempresa) {
        $sql = "SELECT DSDIRLOGOMARCA FROM EMPRESA WHERE CDEMPRESA = :P_CDEMPRESA";
        $params = array(
            "P_CDEMPRESA"   => $cdempresa,
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getNotasLote($cdempresa, $cdinscmuni, $cdinscesta, $nrinsjuriest, $nrseqloterps) {
        $sql = "SELECT NF.CDEMPRESA,
                NF.CDFILIEMIT,
                NF.CDFILIAL,
                NF.IDENTRSAIDOP,
                NF.NRORG,
                NF.CDCLIENTE,
                NF.NRLANCTONF,
                NF.NRNOTAFISC,
                RPS.CDVERIFNFSE,
                E.NMRAZSOCEMPR,
                C.NRINSJURCLIE,
                C.NMRAZSOCCLIE,
                NF.DTEMISSAO,
                E.DSEMAILEMPRESA
           FROM NOTAFISCAL NF
          INNER JOIN INSCRESTAD IE
             ON NF.CDEMPRESA  = IE.CDEMPRESA
            AND NF.CDINSCESTA = IE.CDINSCESTA
          INNER JOIN INSCRESTAD IE
             ON NF.CDEMPRESA  = IE.CDEMPRESA
            AND NF.CDINSCESTA = IE.CDINSCESTA
          INNER JOIN EMPRESA E
             ON NF.CDEMPRESA = E.CDEMPRESA
          INNER JOIN CLIENTE C
             ON NF.CDCLIENTE = C.CDCLIENTE
          INNER JOIN LOTERPS LOTE
             ON NF.CDEMPRESA    = LOTE.CDEMPLOTERPS
            AND NF.NRSEQLOTERPS = LOTE.NRSEQLOTERPS
            AND NF.CDINSCMUNI   = LOTE.CDINSCMUNI
            AND NF.CDINSCESTA   = LOTE.CDINSCESTA
            AND IE.NRINSJURIEST = LOTE.NRINSJURIEST
          INNER JOIN NUMERORPS RPS
             ON NF.CDEMPRESA    = RPS.CDEMPLOTERPS
            AND NF.NRSEQLOTERPS = RPS.NRSEQLOTERPS
            AND NF.NRRPS        = RPS.NRRPS
            AND NF.CDINSCMUNI   = RPS.CDINSCMUNI
            AND NF.CDINSCESTA   = RPS.CDINSCESTA
            AND IE.NRINSJURIEST = RPS.NRINSJURIEST
          WHERE LOTE.CDEMPLOTERPS = :P_CDEMPLOTERPS
            AND LOTE.CDINSCMUNI   = :P_CDINSCMUNI
            AND LOTE.CDINSCESTA   = :P_CDINSCESTA
            AND LOTE.NRINSJURIEST = :P_NRINSJURIEST
            AND LOTE.NRSEQLOTERPS = :P_NRSEQLOTERPS";

        $params = array(
            "P_CDEMPLOTERPS" => $cdempresa,
            "P_CDINSCMUNI"   => $cdinscmuni,
            "P_CDINSCESTA"   => $cdinscesta,
            "P_NRINSJURIEST" => $nrinsjuriest,
            "P_NRSEQLOTERPS" => $nrseqloterps
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    //Passe # na mascara para substituir por numeros. Ex: CPF = "###.###.###-##".
    public static function aplicarMascaraNumerica($mask, $str) {
        $str = str_replace(" ", "", $str);
        for($i = 0; $i < strlen($str); $i++){
            $mask[strpos($mask,"#")] = $str[$i];
        }
        return $mask;
    }

    public function getLocalServidor($cdempresa, $cdinscmuni, $nrorg) {
        return $this->paramNfseService->getLocalServidor($cdempresa, $cdinscmuni, $nrorg);
    }

    public function aprovarLotesAceitos(Request\DataSet $request, Response $response){
        try {
            $lotes = $request->getDataSet()->getRows();
            if($lotes){
                $cdempresa = $lotes[0]["CDEMPLOTERPS"];
                $nrorg = $this->environment->getNrOrg();
        
                if(Util::utilizaAprovacaoAutomatica($this->connection, $cdempresa, $nrorg) === 'S'){
                    foreach($lotes as $lote){
                        $cdempresa    = $lote["CDEMPLOTERPS"];
                        $cdinscmuni   = $lote["CDINSCMUNI"];
                        $cdinscesta   = $lote["CDINSCESTA"];
                        $nrinsjuriest = $lote["NRINSJURIEST"];
                        $nrseqloterps = $lote["NRSEQLOTERPS"];
                        $errosAprovacao = array();
                        $notas = $this->getNotasLote($cdempresa, $cdinscmuni, $cdinscesta, $nrinsjuriest, $nrseqloterps);
                        
                        foreach($notas as $nota){
                            $erro = $this->aprovacaoLancamentosService->aprovar($nota['NRORG'], $nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $response);

                            foreach($erro as $e) {
                                $e["NRLANCTONF"] = $nota["NRLANCTONF"];
                                array_push($errosAprovacao, $e);
                            }
                        }
                    }
                    $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosAprovacao', $errosAprovacao));
                }
                else {
                    $this->errosAprovacao = true;
                    $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('naoAprovaAutomatico', array($this->errosAprovacao)));
                }
            }
        }
		catch (\Exception $e) {
			$response->setError(new Response\Error($e->getMessage(), $e->getCode()));
		}
    }
    
    public function retornaEmailEmpresa ($notas){
        $email =  Util::getEmailEmpresa($this->connection, $this->environment->getNrOrg(), $notas['CDFILIEMIT']);
        if(count($email)>0){
            
            if( $email[0]['DSLOGINSMTP'] ){
                return $email[0]['DSLOGINSMTP'];
            }else if( $email[0]['DSEMAILEMXMLNFE'] ){
                return $email[0]['DSEMAILEMXMLNFE'];
            }else if( $email[0]['DSEMAILFILI'] ){
                return $email[0]['DSEMAILFILI'];
            }else{
                return $notas["DSEMAILEMPRESA"];
            }  
        }else{
            return $notas["DSEMAILEMPRESA"];
        }
    }
    
    public  function retornaAssuntoEmail ($assunto, $nrnotafisc, $dataEmissao, $emitente = ''){
        if($assunto){
            $partesData  = explode('/',$dataEmissao);
            $nomeMes     = $this->retornaMes($partesData[1]); 
            $msg         = str_replace("[MES]",$nomeMes,$assunto);
            return $msg;
        }else{
            return "NFS-e n.".$nrnotafisc." autorizada pelo emitente ".$emitente.".";
        }
    }
    
    public function retornaMes ($numeroMes){
       $mes = array('01' => "Janeiro", 
                    '02' => "Fevereiro", 
                    '03' => "Março",
                    '04' => "Abril", 
                    '05' => "Maio",
                    '06' => "Junho",
                    '07' => "Julho", 
                    '08' => "Agosto", 
                    '09' => "Setembro", 
                    '10' => "Outubro", 
                    '11' => "Novembro", 
                    '12' => "Dezembro");
    
        return $mes[$numeroMes];
    }
    
    public function retornaCorpoEmail ($mensagem ,$nrnotafisc, $cdverifnfse, $razaoSocialPrestador, $razaoSocialTomador, $dataEmissao, $inscricaoPrestador, $inscricaoTomador, $logoempresa = '', $cdPrefeitura, $prefeitura) {
        $partesData      = explode('/',$dataEmissao);
        $nomeMes         = $this->retornaMes($partesData[1]); 
        $mensagem        = str_replace("[MES]",$nomeMes,$mensagem);
        $prefeitura      = strtolower($prefeitura);
        $prefeitura      = ucwords($prefeitura);
        
        if($cdPrefeitura == '6291') $urlPrefeitura = "https://nfse.campinas.sp.gov.br/NotaFiscal/consultarRps.php";
        else if($cdPrefeitura == '4123') $urlPrefeitura = "https://bhissdigital.pbh.gov.br/nfse/pages/consultaNFS-e_cidadao.jsf";
        else if($cdPrefeitura == '4895') $urlPrefeitura = "http://200.149.10.117:8092/iss-web/autenticidade/nfe.xhtml";
        else $urlPrefeitura = '';
        
        if($inscricaoPrestador == '26.269.316/0001-77') $logoempresa = 'https://www.teknisa.com/imagens/logo-teknisa.png';

        if(!$mensagem){
            $prefix          = '<meta http-equiv="Content-Type" content="text/html;charset=utf-8"/><div style="color:#000;font-weight:normal;font-style:normal;text-decoration:none;font-family:Helvetica,Arial,sans-serif;font-size:12pt;"><br><style>table{border:1px solid #cccaca;width:490px;background-color:#dbdbdb}p,pre,td{font-family:Verdana,Geneva,sans-serif !important;color:#000;font-size:14px;line-height:18px}.description{padding:4.2%;padding-bottom:2.1%;padding-top:4px}.title{padding:4.2%;padding-top:2.1%;padding-bottom:0%}.title div{font-weight:bold;width:100%;border-bottom:1px solid #000}.footer p{color:#00528C;font-weight:300;margin-bottom:8.4%}.header .logo{padding-bottom:2.1%}.logo img{width:42%;margin-left:4.2%;margin-top:3.05%}.divTable{display:table;width:100%}.divTable div{display:table-cell}</style><center><table cellpadding="0" cellspacing="0">';
            if($logoempresa) $prefix.='<tr class="header"><td class="logo"><p> <img dfsrc="'.$logoempresa.'" src="'.$logoempresa.'"></p></td></tr>';
            $htmlMensagem    = $prefix.'<tr><td class="title description" style="padding-bottom: 2.1%;"><font face="verdana, helvetica, sans-serif" size="2">Prezado Cliente,<br><br>Este e-mail é um comprovante da emissão da NFS-e número [NRNOTAFISC] autorizada pelo emitente [NMRAZSOCEMIT]. <br><br>A NFS-e é um documento de existência apenas digital, emitido e armazenado eletronicamente com o intuito de documentar uma prestação de serviços. <br><br>Você está recebendo anexo o ESPELHO DA NFS-e mas, se preferir, você poderá imprimir a NFS-e diretamente no site oficial da prefeitura utilizando o Código de Verificação ou de Autenticidade destacado abaixo.<br><br></font><font size="2">Também está anexo o</font><font face="verdana, helvetica, sans-serif" size="2">&nbsp;BOLETO para pagamento do serviço. Basta escanear o código de barras ou digitar os números da linha digitável.</font></td></tr><tr><td class="title"><div>Dados da NFS-e</div></td></tr><tr><td class="description"><div class="divTable"><div style="float: left;width: 100%;"><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline;">Emitente: </font> <font size="2" style="display:inline">[INSCRICAOPRE]</font> <font size="2" style="margin-left: 16.5%;display:block">[NMRAZSOCEMIT]</font></div><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline"><br>Destinatário: </font> <font size="2" style="display:inline">[INSCRICAOCLI]</font> <font size="2" style="margin-left: 19.5%;display:block">[NMRAZSOCCLIE]</font> <font size="2"><br></font></div><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline">Número: </font> <font size="2" style="display:inline">[NRNOTAFISC]</font></div><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline">Dt.Emissão: </font> <font size="2" style="display:inline">[DTEMISSAO]</font></div></div></div></td></tr><tr><td class="title"><div>Autenticidade / Verificação</div></td></tr><tr><td class="description"><div class="divTable"> <font size="2" style="display: inline;">Prefeitura: </font> <font size="2" style="display: inline;">[NMPREFEITURA]</font> <br> <font size="2" style="display: inline">Código: </font> <font size="2" style="display: inline;">[AUTENTICACAO]</font> <br> <font size="2" style="display: inline">Site: </font> <font style="display: inline; font-size: 11.5px"><a href="[URLPREFEITURA]">[URLPREFEITURA]</font></div> <br></td></tr><tr class="footer"><td><center> <br><p>Email automático enviado pelo SM - Service Management.</p></center></td></tr></tbody></table></center></div>';
        }
        else {
            $prefix          = '<meta http-equiv="Content-Type" content="text/html;charset=utf-8"/><div style="color:#000;font-weight:normal;font-style:normal;text-decoration:none;font-family:Helvetica,Arial,sans-serif;font-size:12pt;"><br><style>table{border:1px solid #cccaca;width:490px;background-color:#dbdbdb}p,pre,td{font-family:Verdana,Geneva,sans-serif !important;color:#000;font-size:14px;line-height:18px}.description{padding:4.2%;padding-bottom:2.1%;padding-top:4px}.title{padding:4.2%;padding-top:2.1%;padding-bottom:0%}.title div{font-weight:bold;width:100%;border-bottom:1px solid #000}.footer p{color:#00528C;font-weight:300;margin-bottom:8.4%}.header .logo{padding-bottom:2.1%}.logo img{width:42%;margin-left:4.2%;margin-top:3.05%}.divTable{display:table;width:100%}.divTable div{display:table-cell}</style><center><table cellpadding="0" cellspacing="0">';
            if($logoempresa) $prefix.='<tr class="header"><td class="logo"><p> <img dfsrc="'.$logoempresa.'" src="'.$logoempresa.'"></p></td></tr>';
            $htmlMensagem = $prefix.'<tr><td class="title description" style="padding-bottom: 2.1%;"><font face="verdana, helvetica, sans-serif" size="2">'.$mensagem.'</td></tr>.<tr><td class="title"><div>Dados da NFS-e</div></td></tr><tr><td class="description"><div class="divTable"><div style="float: left;width: 100%;"><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline;">Emitente: </font> <font size="2" style="display:inline">[INSCRICAOPRE]</font> <font size="2" style="margin-left: 16.5%;display:block">[NMRAZSOCEMIT]</font></div><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline"><br>Destinatário: </font> <font size="2" style="display:inline">[INSCRICAOCLI]</font> <font size="2" style="margin-left: 19.5%;display:block">[NMRAZSOCCLIE]</font> <font size="2"><br></font></div><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline">Número: </font> <font size="2" style="display:inline">[NRNOTAFISC]</font></div><div class="divTable" style="display: table;width: 100%;"> <font size="2" style="display: inline">Dt.Emissão: </font> <font size="2" style="display:inline">[DTEMISSAO]</font></div></div></div></td></tr><tr><td class="title"><div>Autenticidade / Verificação</div></td></tr><tr><td class="description"><div class="divTable"> <font size="2" style="display: inline;">Prefeitura: </font> <font size="2" style="display: inline;">[NMPREFEITURA]</font> <br> <font size="2" style="display: inline">Código: </font> <font size="2" style="display: inline;">[AUTENTICACAO]</font> <br> <font size="2" style="display: inline">Site: </font> <font style="display: inline; font-size: 11.5px"><a href="[URLPREFEITURA]">[URLPREFEITURA]</font></div> <br></td></tr><tr class="footer"><td><center> <br><p>Email automático enviado pelo SM - Service Management.</p>';
        }
        
        $replaceMnemonico = array(
            "[NRNOTAFISC]"       => ($nrnotafisc) ? $nrnotafisc : '', 
            "[NMRAZSOCEMIT]"     => ($razaoSocialPrestador) ? $razaoSocialPrestador : '',
            "[INSCRICAOPRE]"     => ($inscricaoPrestador)   ? $inscricaoPrestador : '',
            "[NMRAZSOCCLIE]"     => ($razaoSocialTomador)   ? $razaoSocialTomador : '',
            "[INSCRICAOCLI]"     => ($inscricaoTomador)     ? $inscricaoTomador : '',
            "[DTEMISSAO]"        => ($dataEmissao)          ? $dataEmissao : '',
            "[NMPREFEITURA]"     => ($prefeitura)           ? $prefeitura  : '',
            "[BRASIL]"           => ($cdverifnfse)          ? $cdverifnfse : '',
            "[AUTENTICACAO]"     => ($cdverifnfse)          ? $cdverifnfse : '',
            "[URLPREFEITURA]"    => ($urlPrefeitura)        ? $urlPrefeitura : ''
        );
        $mensagem = strtr($htmlMensagem, $replaceMnemonico); 

        return $mensagem;
    }
    
    
    private function alteraStatusEmail($nrlanctonf, $cdfilial, $nrorg) {
        $this->consultaLoteService->alteraStatusEmail($nrlanctonf, $cdfilial, $nrorg);
    }
    
    private function getLinkCertificado($cdempresa, $cdinscmuni, $nrorg) {
        $certKey = null;
        $certificado = null;
        $fileserverlink = $this->fileServerBaseUrl;
        $certificados = Util::getDataSetCertificadoFileServer($this->connection, $cdinscmuni, $cdempresa, $nrorg, $fileserverlink);
        foreach($certificados as $cert) {
            $extension = explode(".", $cert['fileName']);
            $extension = end($extension);
            if($extension === 'pfx' || $extension === 'p12') $certificado  = $cert['path'];
            else if($extension === 'pem') $certKey = $cert['path'];
        }
        if(is_null($certificado)) throw new \Exception('Certificado não encontrado.');
        else if(is_null($certKey))     throw new \Exception('CertKey não encontrado.');
        else {
            return array("CERTIFICADO" => $certificado, "CERTKEY" => $certKey);
        }
    }
    
}

?>