<?PHP 

namespace Teknisa\DF\Services\NFSe;

class SoapCall {
    
    const URL_PRODUCAO = "https://nfews.prefeitura.sp.gov.br/lotenfeasync.asmx?WSDL";
    const URL_PRODUCAO_SYNC= "https://nfe.prefeitura.sp.gov.br/ws/lotenfe.asmx?WSDL";
    
    public function setCaminhoCertificado($certUploadDir) {
	    if (!is_null($certUploadDir)) {
	        $certUploadDir .= "org_". $this->environment->getNrOrg() . DIRECTORY_SEPARATOR;
	        $this->caminhoCertificado = $certUploadDir;
	       if (!is_dir($this->caminhoCertificado)) {
	            mkdir($this->caminhoCertificado, 0755, true);
	            chmod($this->caminhoCertificado, 0755);
	        }
	    }
	}
    public function assinaXml(\DOMDocument $domXml, array $tags){
        $local = ($this->caminhoCertificado);
        if($local) {
            $dir = $this->caminhoCertificado . $this->cdempresa . "_" . $this->cdinscmuni;
           //$dir = "/home/developer/workfolder/erp/modules/teknisa/df/src/Teknisa/DF/Modules/Saida/NFSe/Certificados/org_1/01_5557301";
            if(!is_dir($dir)) return false;
            if((!is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) && (!is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12"))) return false;
            else {
               
                if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.pfx")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.pfx");
                else if(is_file($dir . DIRECTORY_SEPARATOR . "certificado.p12")) $data = file_get_contents($dir . DIRECTORY_SEPARATOR . "certificado.p12");
            }
        }
        else {
            try{
                $data = file_get_contents($this->linkCertificado);
            }
            catch(\Exception $e) {
                return false;
            }
            if(empty($data)) return false;
        }
        openssl_pkcs12_read($data, $cert, $this->senhaCertificadoDigital);

        if (is_null($cert)) {
            return false;
        }
        else {
            //file_put_contents($dir.DIRECTORY_SEPARATOR."certKey.pem", $cert["pkey"]."\r\n".$cert["cert"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."priKey.pem", $cert["pkey"]);
            //file_put_contents($dir.DIRECTORY_SEPARATOR."pubKey.pem", $cert["cert"]);

            $this->publicKey = $this->getPublicKey($cert["cert"]);
            $this->privateKeyId = openssl_get_privatekey($cert["pkey"]);

            foreach ($tags as $tag) {
                $domXml = $this->assinaXmlAux($domXml, $tag);
            }

            openssl_free_key($this->privateKeyId);
            return $domXml;
        }

    }
    public function getPublicKey($key){
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }
    public function enviarParaWebservice($nomeFuncao, $conteudo) {
        if($this->caminhoCertificado) {
            $caminho = $this->caminhoCertificado . $this->cdempresa . "_" . $this->inscricaoMunicipalTomador . DIRECTORY_SEPARATOR . "certKey.pem";
            $unlink  = false;
        }
        else {
            $certKey = $this->linkCertKey;
            $hash    = uniqid();
            if (is_dir("/tmp")) {
                $caminho= "/tmp/certKey$hash.pem";
            }
            else {
                $caminho = __DIR__ . DIRECTORY_SEPARATOR ."certKey$hash.pem";
            }
            file_put_contents($caminho, file_get_contents($certKey));
            $unlink = true;
        }

       $conteudo = str_replace( "<", "&lt;", $conteudo);
       $conteudo = str_replace( ">", "&gt;", $conteudo);
       $lotUri = self::URL_PRODUCAO_SYNC;     
       $conteudo = $this->createSoapText($conteudo, $nomeFuncao);
       $soapVar = new \SoapVar($conteudo, XSD_ANYXML);
            $resultado = $this->buildSoapCall(
            $caminho,
            $lotUri,
            $lotUri,
            $nomeFuncao,
            $soapVar,
            $conteudo
        );
  
        if($unlink) unlink($caminho);
        return $resultado->outputXML;
    }
    
    public function buildSoapCall($cert, $location, $wsdl, $functionName, $soapVar, $xmlLote) {
        try {
            $msgSize = strlen($xmlLote);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                'SOAPAction: "http://www.prefeitura.sp.gov.br/nfe/ws/consultaNFeRecebidas"',
                "Content-length: $msgSize"
            ];
            $oCurl = curl_init();
            curl_setopt($oCurl, CURLOPT_URL, $wsdl);
            curl_setopt($oCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
            curl_setopt($oCurl, CURLOPT_HEADER, 1);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 0);
            curl_setopt($oCurl, CURLOPT_SSLCERT, $cert);
            curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($oCurl, CURLOPT_POST, true);
            curl_setopt($oCurl, CURLOPT_POSTFIELDS, $xmlLote);
            curl_setopt($oCurl, CURLOPT_HTTPHEADER, $parameters);
          
            $response = curl_exec($oCurl);
            $this->soaperror = curl_error($oCurl);
            $ainfo = curl_getinfo($oCurl);
            if (is_array($ainfo)) {
                $this->soapinfo = $ainfo;
            }
            $headsize = curl_getinfo($oCurl, CURLINFO_HEADER_SIZE);
            $httpcode = curl_getinfo($oCurl, CURLINFO_HTTP_CODE);
            curl_close($oCurl);
            $responseHead = trim(substr($response, 0, $headsize));
            $responseBody = trim(substr($response, $headsize));
            if ($httpcode == 404) {
                throw new \Exception('Erro 404 ao comunicar com a Sefaz.', 1);
            } else if ($httpcode == 500) {
                throw new \Exception($responseBody, 1);
            } else if ($httpcode != 200) {
                throw new \Exception('Erro inesperado.', 1);
            } else{
                    $inicioResponse = strpos($responseBody, "<RetornoXML>");
                    $length         = strrpos($responseBody, "</RetornoXML>") - $inicioResponse + 13;
                    $responseBody   = substr($responseBody, $inicioResponse, $length);
                    $responseBody   = str_replace("&lt;", "<", $responseBody);
                    $responseBody   = str_replace("&gt;", ">", $responseBody);
                    $response       = new \stdClass();
                    $response->outputXML = $responseBody;
                    return $response;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function assinaXmlAux(\DOMDocument $domXml, $tag) {
        $nodes = $domXml->getElementsByTagName($tag);
        
        foreach ($nodes as $node) {
            $root = $node; 
            $id = trim($node->getAttribute("Id"));
            $dados = $node->C14N(false, false, null, null);
            $hashValue = hash("sha1", $dados, true);
            $digValue = base64_encode($hashValue);
            $signature = $domXml->createElement("Signature");
            $root->appendChild($signature);
            $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
        
            $signedInfo = $domXml->createElement("SignedInfo");
            $signature->appendChild($signedInfo);
        
            $novaTag = $domXml->createElement("CanonicalizationMethod");
            $signedInfo->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");
        
            $novaTag = $domXml->createElement("SignatureMethod");
            $signedInfo->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");
        
            $reference = $domXml->createElement("Reference");
            $signedInfo->appendChild($reference);
            $reference->setAttribute("URI", "");
        
            $transforms = $domXml->createElement("Transforms");
            $reference->appendChild($transforms);
        
            $novaTag = $domXml->createElement("Transform");
            $transforms->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");
        
            $novaTag = $domXml->createElement("Transform");
            $transforms->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");
        
            $novaTag = $domXml->createElement("DigestMethod");
            $reference->appendChild($novaTag);
            $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");
        
            $novaTag = $domXml->createElement("DigestValue", $digValue);
            $reference->appendChild($novaTag);
        
            $dados = $signedInfo->C14N(false, false, null, null);
            
        
            $assinatura = "";
            openssl_sign($dados, $assinatura, $this->privateKeyId);
            $signatureValue = base64_encode($assinatura);
            $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
            $signature->appendChild($novaTag);
        
            $keyInfo = $domXml->createElement("KeyInfo");
            $signature->appendChild($keyInfo);
        
            $X509Data = $domXml->createElement("X509Data");
            $keyInfo->appendChild($X509Data);
        
            $novaTag = $domXml->createElement("X509Certificate", $this->publicKey);
            $X509Data->appendChild($novaTag);
        }
        
        return $domXml;
    }
    
}